
#################################################################
# Script for yield data analysis for LPJmL4 paper
# written by C. Müller
# based on GGCMI aggregation and detrending
# extended by J. Breier 
#   to loop over crops using mask mirca due to data lack in spam
#################################################################

# libraries ####
require(fields)
require(ncdf4)
require(zoo)

# functions ####

read.LPJmL.grid <- function(filename,npix,header,swap=F){
  ff <- file(filename,"rb")
  seek(ff,where=header,origin="start")
  if(swap){
    x <- readBin(ff,integer(),size=2,n=npix*2,endian="swap")/100
  } else {
    x <- readBin(ff,integer(),size=2,n=npix*2)/100
  }
  lon <- x[(1:npix)*2-1]
  lat <- x[(1:npix)*2]
  # ilon/ilat are row and column numbers, starting at 179.75W and 89.75N as 1/1
  ilon <- as.integer((lon+180)/0.5 + 1.01)
  ilat <- as.integer((-lat+90)/0.5 + 1.01)
  area <- (111e3*0.5)*(111e3*0.5)*cos(lat/180*pi)/10000
  close(ff)
  data.frame(lon,lat,ilon,ilat,area)
}

read.LPJmL.matrix <- function(path,fn,fy,sy,ey,si,nbands,npix=67420){
  ny <- (ey-sy+1)
  ff <- file(paste0(path,fn),"rb")
  seek(ff,where=(sy-fy)*nbands*npix*si,origin="start")
  data <- readBin(ff,if(si==2) integer() else double(),size=si,n=ny*nbands*npix)
  close(ff)
  dim(data) <- c(npix,nbands,ny)
  data
}

read.ncdf.mask <- function(path,fn){
  nf <- nc_open(paste0(path,fn))
  data <- ncvar_get(nf,varid=names(nf$var)[1])
  nc_close(nf)
  data
}
read.ncdf.mask2 <- function(path,fn){
  nf <- nc_open(paste0(path,fn))
  ir <- ncvar_get(nf,varid="irrigated")
  rf <- ncvar_get(nf,varid="rainfed")
  nc_close(nf)
  list(irrigated=ir,rainfed=rf)
}

read.ncdf.mask3 <- function(path,fn,crops,watersupply){
  nf <- nc_open(paste0(path,fn))
  crops.ext <- intersect(names(nf$var),crops)
  for (c in crops.ext){
    crop.mirca <- ncvar_get(nf,varid=c)
    assign(paste0(watersupply,"_",c,"_mirca"),crop.mirca,pos=+1)
  }
  nc_close(nf)
}
detrend <- function(x1,L){
  #
  t_x1 <- rollapply(c(rep(NA,(L-1)),x1),L,mean,na.rm=T)
  x1-t_x1
}

detrended_cor <- function(x1,x2){
  # check for computation of correlation coefficient | p-value
  x12 <- x1*x2
  if(length(x12[which(is.finite(x12))])<3 | sd(x1,na.rm=T)==0 | sd(x2,na.rm=T)==0)
    return(c(NA,NA))
  x1_dt <- detrend(x1,3)[-1]
  x2_dt <- detrend(x2,3)[-1] 
  x12 <- x1_dt*x2_dt
  if(length(x12[which(is.finite(x12))])<3)
    return(c(NA,NA))
  co <- cor.test(x1_dt,x2_dt, use ="na.or.complete")
  list(x1_dt,x2_dt,as.numeric(co$estimate),co$p.value)
}
read.LPJmL.yield <- function(filename,fyear,syear,eyear,nbands,npix){
  ff <- file(filename,"rb")
  yield <- array(NA,dim=c(npix,nbands,(eyear-syear+1)))
  for(y in syear:eyear){
    for(b in 1:nbands){
    seek(ff,where=((y-fyear)*32+(b-1))*4*npix,origin="start")
    yield[,b,(y-syear+1)] <- readBin(ff,double(),size=4,n=npix)
    }
  }
  close(ff)
  yield
}


# settings ####

ncell <- 67420
# data.path <- "/p/projects/open/sibyll/LPJ_newphen_newgrass/LPJ_newphen_edit/output_oldpar_monthly/"
#data.path <- "/p/projects/open/sibyll/LPJ_newphen_newgrass/LPJ_newphen_edit/crop_runs/yields/GGCMI2_outputs/AgMERRA_default/"
data.path.trunk <- "/p/projects/macmit/users/cmueller/nitrogen/trund_no_n/allcrops/"
#data.path.trunk <- "/p/projects/biodiversity/bloh/nitrogen/new_ws/output/"
data.path <- "/p/projects/macmit/users/cmueller/nitrogen/allcrops/bnf/"
data.path <- "/p/projects/macmit/data/GGCMI/CTWN/agmerra_fullharm_ctwnpar/"
# data.path <- "/p/projects/isimip/isimip-lpj/yields/GGCMI2_outputs/AgMERRA_default/"
mask.path <- "/p/projects/macmit/data/GGCMI/AgMIP.output/processed/masks/"
ref.path <- "/p/projects/macmit/data/GGCMI/AgMIP.input/other.inputs/reference/faostat/"
# fig.path <- "/p/projects/macmit/users/cmueller/LPJmL4/yield_variability/"
#fig.path <-"/home/breier/eval_YIELD_Var/"
fig.path <- data.path

# freshmatter values in sequence of LPJmL crops (whe, ric, mai, etc.)
FRESHMATTER <- 100 / c(88, 87, 88, 88, 90, 24, 35, 93, 91, 94, 92, 27, 100, 100)  # cassava 


# select crops
# crops.mirca <- c("Wheat","Rice","Maize","Millet","Sugar_beet","Cassava","Sunflower",
#                  "Soy","Rapeseed","Sugarcane") # "Pulses"/"peas_dry"?, Soy=Soybean?, "Groundnut"?
crops.mirca <- c("Wheat","Rice","Maize","Soy")

# crops.ref.mirca <- c("yield_whe","yield_ric","yield_mai", "yield_mil","yield_sgb","yield_cassava", 
#                      "yield_sunflower","yield_soy","yield_rap","yield_sug")
crops.ref.mirca <- c("yield_whe","yield_ric","yield_mai", "yield_soy")

gadm0.crops <- c("gadm0.whe","gadm0.ric","gadm0.mai","gadm0.soy") # has to be harmonized with crops.mirca & crops.ref.mirca

topproducer.crop <- c("topproducer.whe","topproducer.ric","topproducer.mai","topproducer.soy")
# topproducer.crop <- c("topproducer.whe")


cft.order.rf <- c(1,2,3,9)
cft.order.ir  <- c(17,18,19,25)

# cft.order.rf<- c(1,2,3,4,6,7,8,9,11,12)
# cft.order.ir<- c(17,18,19,20,22,23,24,25,27,28)


# select top 10 producers
topproducer.whe <- c("China","India","USA","Russia","France","Canada","Australia","Pakistan","Germany","Turkey")
gadm0.whe <- c(48,105,240,186,79,41,14,170,86,232)

topproducer.soy <- c("USA","Brazil","Argentina","China","India","Paraguay","Canada","Uruguay","Ukraine","Bolivia")
gadm0.soy <- c(240,32,11,48,105,175,41,242,237,27)

topproducer.ric <- c("China","India","Indonesia","Bangladesh","Viet Nam","Thailand","Myanmar","Philippines","Brazil","Japan")
gadm0.ric <- c(48,105,106,19,247,226,154,177,32,114)

topproducer.mai <- c("USA","China","Brazil","Argentina","Mexico","India","Ukraine","Indonesia","France","South_Africa")
gadm0.mai <- c(240,48,32,11,145,105,237,106,79,209)


# main ####

# read grid
grid <- read.LPJmL.grid(paste0(data.path,"grid.bin"),ncell,0)

# read yield data time series
#lpjml.yield <- read.LPJmL.matrix(data.path,"pft_harvest.pft.bin",1901,1961,2011,4,32)
#lpjml.yield <- read.LPJmL.matrix(data.path,"pft_charvest.pft.bin",1901,1980,2010,4,32)
lpjml.yield <- read.LPJmL.matrix(data.path,"pft_harvest.bin",1980,1980,2010,4,32)
lpjml.yield.trunk <- read.LPJmL.matrix(data.path.trunk,"pft_harvest.pft.bin",1901,1980,2010,4,32)

#lpjml.yield <- read.LPJmL.yield(paste0(data.path,"/pft_charvest.pft.bin"),1901,1980,2010,32,67420)
#lpjml.yield.trunk <- read.LPJmL.yield(paste0(data.path.trunk,"/pft_harvest.pft.bin"),1901,1980,2010,32,67420)

# read masks
gadm0 <- read.ncdf.mask(mask.path,"aggr/gadm0.mask.nc4")
gadm0.id <- read.csv(paste0(mask.path,"aggr/gadm0.meta.csv"),header=F)[,1]  
gadm0.name <- read.csv(paste0(mask.path,"aggr/gadm0.meta.csv"),header=F)[,2]  		


# read crops mirca as abs.
read.ncdf.mask3(mask.path,"weight/landuse.rf.nc4",crops.mirca,"rf")
read.ncdf.mask3(mask.path,"weight/landuse.ir.nc4",crops.mirca,"ir")


# aggregate to national values
# try loop over countries, maybe faster with doing it for full matrix? But more memory required...
# create production time series per pixel
for (cr in 1:length(crops.mirca)){
  crop.mirca_rf <- get(paste0("rf_",crops.mirca[cr],"_mirca"))
  crop.mirca_ir <- get(paste0("ir_",crops.mirca[cr],"_mirca"))
  gadm0.crop <- get(gadm0.crops[cr])
  
  crop.prod <- array(NA,dim=c(dim(lpjml.yield)[c(1,3)])) # same pixels & years as in yield data, 2 aggregation masks
  crop.prod.trunk <- array(NA,dim=c(dim(lpjml.yield.trunk)[c(1,3)])) # same pixels & years as in yield data, 2 aggregation masks
  for(p in 1:ncell){
    crop.prod[p,] <- lpjml.yield[p,cft.order.rf[cr],]*crop.mirca_rf[grid$ilon[p],grid$ilat[p]]*grid$area[p]/0.45/100*FRESHMATTER[cft.order.rf[cr]] + 
      lpjml.yield[p,cft.order.ir[cr],]*crop.mirca_ir[grid$ilon[p],grid$ilat[p]]*grid$area[p]/0.45/100*FRESHMATTER[cft.order.rf[cr]] 
    crop.prod.trunk[p,] <- lpjml.yield.trunk[p,cft.order.rf[cr],]*crop.mirca_rf[grid$ilon[p],grid$ilat[p]]*grid$area[p]/0.45/100*FRESHMATTER[cft.order.rf[cr]]  + 
      lpjml.yield.trunk[p,cft.order.ir[cr],]*crop.mirca_ir[grid$ilon[p],grid$ilat[p]]*grid$area[p]/0.45/100*FRESHMATTER[cft.order.rf[cr]] 
    #crop.mirca.rf <- crop.mirca_rf*grid$area[p]
    #crop.mirca.ir <- crop.mirca_ir*grid$area[p]
  }
  

  
  # create national production time series
  crop.national.prod <- array(NA,dim=c(length(gadm0.id),dim(lpjml.yield)[3])) # countries, years as in yield data, 2 aggregation masks
  crop.national.prod.trunk <- array(NA,dim=c(length(gadm0.id),dim(lpjml.yield.trunk)[3])) # countries, years as in yield data, 2 aggregation masks
  crop.national.area <- array(NA,dim=c(length(gadm0.id))) # countries, 2 aggregation masks
  for(nat in 1:length(gadm0.id)){
    # find pixels that belong to this country (nat)
    pix <-  which(gadm0==gadm0.id[nat],arr.ind=T)
    if(length(pix>0)){
      crop.national.prod[nat,] <- 0
      crop.national.prod.trunk[nat,] <- 0
      crop.national.area[nat] <- 0
      for(pp in 1:dim(pix)[1]){
        pin <- which(grid$ilon==pix[pp,1] & grid$ilat==pix[pp,2])
        #wheat.national.prod[nat,,1] <- wheat.national.prod[nat,] + ifelse(is.finite(wheat.prod[pin,]),crop.prod[pin,],0)
        crop.national.prod[nat,] <- crop.national.prod[nat,] + ifelse(is.finite(crop.prod[pin,]),crop.prod[pin,],0)
        crop.national.prod.trunk[nat,] <- crop.national.prod.trunk[nat,] + ifelse(is.finite(crop.prod.trunk[pin,]),crop.prod.trunk[pin,],0)
        crop.national.area[nat] <- crop.national.area[nat] + sum(crop.mirca_rf[pix[pp,1],pix[pp,2]]+crop.mirca_ir[pix[pp,1],pix[pp,2]],na.rm=T)*grid$area[pin]
      }
    }
  }
  
  # read reference data
  nf <- nc_open(paste0(ref.path,"faostat.1961-2012.gadm0.fixed_mirca_mask.nc4"))
  gad <- ncvar_get(nf,varid="gadm0")
  #ref <- ncvar_get(nf,varid="yield_whe")[1,1,1:51,]
  ref0 <- ncvar_get(nf,varid=crops.ref.mirca[cr])[1,1,20:50,]# for agmerra
  refp1 <- ncvar_get(nf,varid=crops.ref.mirca[cr])[1,1,21:51,]# for agmerra
  refm1 <- ncvar_get(nf,varid=crops.ref.mirca[cr])[1,1,19:49,]# for agmerra
  nc_close(nf)
  
  
  png(paste0(fig.path,crops.mirca[cr],"_ts_topproducer_ctwn_sdate_and_phu5.png"),
       width=8*300,height=5*300,res=300,pointsize=8,
       type="cairo")
  #pdf(paste0(fig.path,crops.mirca[cr],"_ts_topproducer_agmerra.pdf"))
  # x11()
  par(oma=c(0,0,0,0))
  split.screen(c(3,4))
  
  for(tp in 1:length(gadm0.crop)){
    y <- crop.national.prod[which(gadm0.id==gadm0.crop[tp]),]/
      crop.national.area[which(gadm0.id==gadm0.crop[tp])]
    test0 <- detrended_cor(y,ref0[,which(gad==gadm0.crop[tp])])
    testp1 <- detrended_cor(y,refp1[,which(gad==gadm0.crop[tp])])
    testm1 <- detrended_cor(y,refm1[,which(gad==gadm0.crop[tp])])
    y.trunk <- crop.national.prod.trunk[which(gadm0.id==gadm0.crop[tp]),]/
      crop.national.area[which(gadm0.id==gadm0.crop[tp])]
    test0.trunk <- detrended_cor(y.trunk,ref0[,which(gad==gadm0.crop[tp])])
    testp1.trunk <- detrended_cor(y.trunk,refp1[,which(gad==gadm0.crop[tp])])
    testm1.trunk <- detrended_cor(y.trunk,refm1[,which(gad==gadm0.crop[tp])])
    
    # move time series only if this improves both time series
    if((test0[[3]] < (testp1[[3]] - 0.2) & (test0.trunk[[3]] < (testp1.trunk[[3]] - 0.2) | test0.trunk[[4]] >= 0.1)) | 
       (test0[[3]] < (testm1[[3]] - 0.2) & (test0.trunk[[3]] < (testm1.trunk[[3]] - 0.2) | test0.trunk[[4]] >= 0.1))){
      test <- if(testp1[[3]] > testm1[[3]]) testp1 else testm1
      test.trunk <- if(testp1[[3]] > testm1[[3]]) testp1.trunk else testm1.trunk
      ref <- if(testp1[[3]] > testm1[[3]]) refp1 else refm1
    } else {
      test <- test0
      test.trunk <- test0.trunk
      ref <- ref0
    }
    screen(tp)
    par(mar=c(2,2,4,2))
    yr <- range(y,y.trunk,ref[,which(gad==gadm0.crop[tp])],
                test[[1]],test[[2]],na.rm=T)
    yr[2] <- yr[2]+sd(yr)*0.2
    ypos <- yr[2]-(yr[2]-yr[1])*0.1
    plot(c(1980:2010),ref[,which(gad==gadm0.crop[tp])],ylim=yr,type="l",
         ylab="",xlab="",main=get(topproducer.crop[cr])[tp], cex.axis=0.75,cex.main=0.9)
    lines(c(1980:2010),y,col=2)
    lines(c(1980:2010),y.trunk,col=3)
    lines(c(1981:2010),test[[2]],lty=2)
    lines(c(1981:2010),test[[1]],col=2,lty=2)
    lines(c(1981:2010),test.trunk[[1]],col=3,lty=2)
    text(1982,ypos,paste(format(test[[3]],nsmall=3,digits=3),
                         if(test[[4]]<0.001) "***" else if(test[[4]]<0.01) "**" else if(test[[4]]<0.05) "*" else if(test[[4]]<0.1) "." else "n.s."),cex=0.6,adj=c(0,0),col=2)
    text(2002,ypos,paste(format(test.trunk[[3]],nsmall=3,digits=3),
                         if(test.trunk[[4]]<0.001) "***" else if(test.trunk[[4]]<0.01) "**" else if(test.trunk[[4]]<0.05) "*" else if(test.trunk[[4]]<0.1) "." else "n.s."),cex=0.6,adj=c(0,0),col=3)
  }
  
  # legend
  screen(11)
  par(mar=c(2,2,4,2),xpd=NA)
  plot(1,type="n",axes=F,ylab="",xlab="")
  legend("topleft",lty=c(1,1,1,2,2,2),col=c(1,2,3,1,2,3),
         legend=c("FAOstat","LPJmL_N","LPJmL_trunk",
                  "FAOstat residuals","LPJmL_N residuals","LPJmL_trunk resid."),
         bty="n",y.intersp=1, cex=0.75)
  # significance levels
  screen(12)
  par(mar=c(2,2,4,2),xpd=NA)
  plot(1,type="n",axes=F,ylab="",xlab="")
  legend("topleft",
         legend=c("*** p<0.001","** p<0.01","* p<0.05",". p<0.1","n.s. not significant"),
         bty="n",y.intersp=1, cex=0.75)
  
  # close.screen(all=T)
  dev.off()

}
