
#-------------------------------------------------------------------
# cal_helper.r
# R-Script prepare LPJ simulations with fixed LAIMAX 1 ... 7 for crop calibration,
# prepare calibration script, and construct new laimax parameter file.
#
# Tim Beringer
# 18.12.2013
#-------------------------------------------------------------------


# calibration workflow ####

# 1. check out trunk from SVN server
# 2. compile code
# 3. use this script to prepare lpj simulations with laimax 1 to 7
# 4. use lpjdistribute.sh to start simulations
# 5. call calibration script
# 6. create new laimax.par file with this script

### PART 1 ###
### create confs, pars and output dirs

closeAllConnections()
rm(list=ls(all=TRUE))
gc()

# SET DIRECTORIES, EMAIL, IDENTITY, ETC. ####

# path to source code from computer running this script
d.base <- c("/iplex/01/waterforce/jonas/Diss/LPJmL_2571_rw/")
# path to FAO calib folder
d.fao <- c("/iplex/01/waterforce/jonas/Diss/LPJmL_2571_rw/FAO_calib/")
# path to source code on cluster, initial dir in jcf file
d.host <- "/iplex/01/waterforce/jonas/Diss/LPJmL_2571_rw/"

# directory used by calibration script, will be created if not existing
d.cal <- c(paste(d.base,"/FAO_calib/output.yields/",sep=""))
d.par <- c(paste(d.base,"par/",sep=""))

# just in case the lpjml executable has been renamed
f.lpj.bin <- "lpjml"
lpj.bin <- paste(d.host,"bin/",f.lpj.bin,sep="")

# just in case standard output directory in lpjml.conf hast already been changed
lpjml.conf.out.dir <- "output_lu/"

# set email for job notification
my.email <- "jonasjae@pik-potsdam.de"
my.name <- "J. Jägermeyr"

# define files for output
f.out <- c("------------------- --- ----------------------------- */",
           "GRID                RAW output/grid.bin",
           "FPC                 RAW output/fpc.bin ",
           "MNPP                RAW output/mnpp.bin",
           "MGPP                RAW output/mgpp.bin",
           "MRH                 RAW output/mrh.bin",
           "MFPAR               RAW output/mfpar.bin",
           "MTRANSP             RAW output/mtransp.bin",
           "MRUNOFF             RAW output/mrunoff.bin",
           "MEVAP               RAW output/mevap.bin",
           "MINTERC             RAW output/minterc.bin",
           "MSWC1               RAW output/mswc1.bin",
           "MSWC2               RAW output/mswc2.bin",
           "FIREC               RAW output/firec.bin",
           "FIREF               RAW output/firef.bin",
           "VEGC                RAW output/vegc.bin",
           "SOILC               RAW output/soilc.bin",
           "LITC                RAW output/litc.bin",
           "FLUX_ESTAB          RAW output/flux_estab.bin",
           "#ifdef WITH_SPITFIRE",
           "MFIREC              RAW output/mfirec.bin",
           "MNFIRE              RAW output/mnfire.bin",
           "MBURNTAREA          RAW output/mburnt_area.bin",
           "#endif",
           "#ifdef RIVER_ROUTING",
           "MDISCHARGE          RAW output/mdischarge.bin",
           "#endif",
           "#ifdef WITH_LANDUSE",
           "HARVEST             RAW output/flux_harvest.bin",
           "SDATE               RAW output/sdate.bin",
           "PFT_HARVEST         RAW output/pft_harvest.SUFFIX",
           "CFTFRAC             RAW output/cftfrac.bin",
           "#endif",
           "/*----------------- --- ------------------------------- */")

f.jcf.ref <- c(
  "#!/bin/ksh",
  "# @ job_type = parallel",
  "# @ total_tasks = 32",
  "# @ class = short",
  "# @ group = open",
  "# @ comment = LPJmL Version 3.5.003 args: -DFROM_RESTART",
  "# @ environment = COPY_ALL",
  "# @ blocking = UNLIMITED",
  "# @ output = /lpjml.out",
  "# @ error = /lpjml.err",
  "# @ notify_user = jonasjae@pik-potsdam.de",
  "# @ notification = complete",
  "# @ initialdir = /",
  "# @ queue",
  "",
  "llgetmachinelist > hostlist.$LOADL_STEP_ID",
  "",
  "cat hostlist.$LOADL_STEP_ID|uniq >mpdhosts.$LOADL_STEP_ID",
  "machine_count=`cat mpdhosts.$LOADL_STEP_ID  | wc -l`",
  "node_count=`cat hostlist.$LOADL_STEP_ID  | wc -l`",
  "export MPD_CON_EXT=$LOADL_STEP_ID",
  "mpdboot -n $machine_count -r ssh -f mpdhosts.$LOADL_STEP_ID",
  "",
  "mpiexec -machinefile hostlist.$LOADL_STEP_ID -n $node_count  /bin/lpjml -DFROM_RESTART lpjml.conf",
  "rc=$?  # save return code of mpiexec",
  "#mpdallexit",
  "rm hostlist.$LOADL_STEP_ID",
  "rm mpdhosts.$LOADL_STEP_ID",
  "exit $rc # exit with return code")

f.param.par <- readLines(paste(d.par,"param.par",sep=""))
pos.param.par <- grep("maximum LAI",f.param.par)
f.param.conf <- readLines(paste(d.base,"param.conf",sep=""))
pos.param.conf <- grep("LPJ parameter file",f.param.conf)
f.lpjml.conf <- readLines(paste(d.base,"lpjml.conf",sep=""))
pos1.lpjml.conf <- grep("BENCHMARK",f.lpjml.conf)
pos2.lpjml.conf <- grep("Input parameter file",f.lpjml.conf)

# edit jcf file ####

# change email for notification
f.jcf.ref[grep("notify_user",f.jcf.ref)] <- 
  paste(unlist(strsplit(f.jcf.ref[grep("notify_user",f.jcf.ref)],"="))[1],"= ", my.email, sep="")
# add LPJROOT definition
f.jcf.ref <- c(f.jcf.ref[1:15],paste("export LPJROOT=",d.host,sep=""),"",f.jcf.ref[16:length(f.jcf.ref)])

# edit lpjml.conf ####

# define BENCHMARK_LAI
f.lpjml.conf[grep("#define BENCHMARK_LAI",f.lpjml.conf)] <- "#define BENCHMARK_LAI"  
# define WITH_LANDUSE
f.lpjml.conf[grep(glob2rx("*#define *LANDUSE*"),f.lpjml.conf)[1]] <- 
  "#define WITH_LANDUSE 1 /*NO_LANDUSE: land use disabled; WITH_LANDUSE: land use enabled */"
# set spinup years
f.lpjml.conf[grep("spinup years",f.lpjml.conf)[3]] <-
  "120 /* spinup years */"
# set first year of simulation
f.lpjml.conf[grep("first year of simulation",f.lpjml.conf)[2]] <-
  "1901 /* first year of simulation */"
# set last year of simulation
f.lpjml.conf[grep("last year of simulation",f.lpjml.conf)[3]] <-
  "2009 /* last year of simulation */"
# do not start from restart
f.lpjml.conf[grep("start from restart file",f.lpjml.conf)[2]] <-
  "NO_RESTART /* start from restart file */"
# uncomment restart filename
f.lpjml.conf[grep("filename of restart file",f.lpjml.conf)[2]] <-
  "/* restart/restart.lpj.dummy*/ /* filename of restart file */"
# create restart file in 1900
f.lpjml.conf[grep("create restart file",f.lpjml.conf)[2]] <-
  "RESTART /* create restart file */"
# set year of restart
f.lpjml.conf[grep("write restart at year",f.lpjml.conf)[1]] <-
  "1900 /* write restart at year; exclude line in case of no restart to be written */"

param.conf.praef <- paste(substr(f.param.conf[pos.param.conf],1,10),d.fao,substr(f.param.conf[pos.param.conf],15,20),sep="")
param.conf.suf <- substr(f.param.conf[pos.param.conf],20,nchar(f.param.conf[pos.param.conf]))

lpjml.conf.praef <- paste(substr(f.lpjml.conf[pos2.lpjml.conf],1,10),d.fao,substr(f.lpjml.conf[pos2.lpjml.conf],11,16),sep="")
lpjml.conf.suf <- substr(f.lpjml.conf[pos2.lpjml.conf],16,nchar(f.lpjml.conf[pos2.lpjml.conf]))

# create lpjml.conf, param.conf, param.par for 7 LAIMAX simulations ####

for(i in 1:7){
  # create param.par.laimax.*.par
  substr(f.param.par[pos.param.par],1,1) <- as.character(i)
  writeLines(f.param.par,paste(d.fao,"param.laimax.",i,".par",sep=""))
  
  # create param.laimax.*.conf
  f.param.conf[pos.param.conf] <- paste(param.conf.praef,"laimax.",i,param.conf.suf,sep="")
  writeLines(f.param.conf,paste(d.fao,"param.laimax.",i,".conf",sep=""))
  
  # create lpjml.laimax.*.conf
  f.lpjml.conf.laimax <- gsub(lpjml.conf.out.dir, paste(d.fao,"output.laimax.",as.character(i),"/",sep=""), f.lpjml.conf)
  f.lpjml.conf.laimax[grep("Input parameter file",f.lpjml.conf.laimax)] <-
    paste(lpjml.conf.praef,"laimax.",i,lpjml.conf.suf,sep="")
  # change output directory
  f.lpjml.conf.laimax[grep("output",f.lpjml.conf.laimax)]
  for(j in 1:length(grep("RAW",f.lpjml.conf.laimax))){
    unlist(strsplit(f.lpjml.conf.laimax[grep("RAW",f.lpjml.conf.laimax)[j]],"/"))
  }
  
  # change output directory
  pos.start <- grep(glob2rx(f.out[1]),f.lpjml.conf.laimax) # find start of output file section
  pos.end <- grep(glob2rx(f.out[34]),f.lpjml.conf.laimax) # find end of output file section
  length.output <- pos.end - pos.start + 1
  if(length.output != length(f.out)) print("WARNING: Check definition of output files in lpjml*.conf!")
  
  for(j in (pos.start+1):(pos.end-1)){
    if(length(unlist(strsplit(f.out[j-pos.start+1],"output/")))==2)
      f.lpjml.conf.laimax[j] <- 
      paste(unlist(strsplit(f.out[j-pos.start+1],"output/"))[1],d.fao,"output.laimax.",i,"/",unlist(strsplit(f.out[j-pos.start+1],"output/"))[2],sep="")
  }
  
  # change name of restart file to be written after 1900
  f.lpjml.conf.laimax[grep("filename of restart file",f.lpjml.conf.laimax)[3]] <- 
    paste("restart/restart_1900_laimax.",i,".lpj /* filename of restart file */",sep="")
  writeLines(f.lpjml.conf.laimax,paste(d.fao,"lpjml.laimax.",i,".conf",sep=""))
  
  # create lpjml.laimax.*.jcf
  lpj.conf.laimax <- paste(d.fao,"lpjml.laimax.",i,".conf",sep="")
  f.jcf.ref[grep("mpiexec -machinefile",f.jcf.ref)]  <-
    paste("mpiexec -machinefile hostlist.$LOADL_STEP_ID -n $node_count ", lpj.bin, " -DFROM_RESTART", lpj.conf.laimax)
  f.jcf.ref[grep("# @ output",f.jcf.ref)] <- 
    paste(unlist(strsplit(f.jcf.ref[grep("# @ output",f.jcf.ref)],"="))[1]," = ", d.fao,"output.laimax.",i,"/lpjml.out",sep="")
  f.jcf.ref[grep("# @ error",f.jcf.ref)] <- 
    paste(unlist(strsplit(f.jcf.ref[grep("# @ error",f.jcf.ref)],"="))[1]," = ", d.fao,"output.laimax.",i,"/lpjml.err",sep="")
  f.jcf.ref[grep("# @ initialdir",f.jcf.ref)] <- 
    paste(unlist(strsplit(f.jcf.ref[grep("# @ initialdir",f.jcf.ref)],"="))[1]," = ", d.fao, sep="")
  
  writeLines(f.jcf.ref,paste(d.fao,"lpjml.laimax.",i,".jcf",sep=""))
  
  # create output directories
  dir.create(paste(d.fao,"output.laimax.",i,sep=""))
}

print("Ready!")

