rm(list=ls(all=TRUE))
gc()

### ------------------------------------------------------------------------------------------------------------ ###
### This script generates a 64 band landuse input based a standard 32 band input 
###
### Irrigated area in each cell is seprated for surface, sprinkler, and drip irrigation based on 
### national shares for each system (read from table Country_irrigation_system_shares_Aquastat_ICID_Rohwer.csv)
### New band order is: 1-16 rain-fed, 17-32 surface irrigation, 33-48 sprinkler, 49-64 drip
### Please note, the script uses random sampling algorithms and thus produces results that are not identiaclly 
### reproducible (JJ 06/2015)
### ------------------------------------------------------------------------------------------------------------- ###

### --------------------------------------------- ###

### set reading path to old 32 band landuse input ### 
32_BAND_LANUSE_INPUT <- "/iplex/01/2012/open/Sibyll/MATTI/ANALYSE/cft1700_2005_pasture.clm"
refyear=1700 # start year of input file
read= c(1700,2005) # first and last year to read from
nband=32

### set writing path to new 64 band landuse input ### 
64_BAND_LANUSE_INPUT <- ".../cft1700_2005_irrigation_systems_64bands.bin"

### --------------------------------------------- ###


library(raster)

### read Grid
ncell <- file.info("/iplex/01/2012/open/DATA/landuse/grid.bin")$size/4
grid <- t(array(readBin("/iplex/01/2012/open/DATA/landuse/grid.bin",what=integer(),size=2,n=ncell*2),dim=c(2,ncell))/100) # lon = grid[,1], lat = grid[,2]
cellwidth <- 6371000.785*pi/180 # earthradius [m] times Pi
cellarea<- (cellwidth*0.5)*(cellwidth*0.5)*cos(grid[,2]/180*pi)

### load LPJmL countrys
c.names<- read.table("/iplex/01/2012/open/DATA/landuse/Country_names.txt",header=TRUE)
zz <- file(paste("/data/biosx/LPJ/cru06/cow_mg_2006.bin",sep=""),"rb")
seek(zz,where=39,origin="start")
x <- readBin(zz, integer(), n=2*ncell, size=2, endian="swap")
close(zz)
lpj.countries <- x[c(1:ncell)*2-1]
lpj.countries[lpj.countries==161]<-  193 # replace Taiwan with values for China

### load FAO country irrigation shares
cntr_shares=read.csv("/iplex/01/2012/open/DATA/landuse/Country_irrigation_system_shares_Aquastat_ICID_Rohwer.csv",header=TRUE)[,c(1,9,10,11)]

cftnames <-c("temperate_cereals","rice","maize","tropical_cereals","pulses","temperate_roots","tropical_roots","sunflower","soybean","groundnut","rapeseed","sugar_cane","others","pastures","bioenergy_grass","bioenergy_tree")

### crop suitability (after Sauer et al. 2010 and Fischer 2012)
suit=data.frame(Index=1:16,CFT=cftnames,
                surf=c(1,1,1,1,1,1,0,1,1,1,1,1,1,1,0,0),
                sprink=c(1,0,1,1,1,1,0,1,1,1,1,1,1,1,0,0),
                drip=c(0,0,0,0,1,0,0,1,1,0,0,0,1,0,0,0))

surf_suit=suit[,3]
sprink_suit=suit[,4]
drip_suit=suit[,5]

SURF=1
SPRINK=2
DRIP=3
SURFBANDS=c(1:16)
SPRINKBANDS=c(17:32)
DRIPBANDS=c(33:48)

### ------------------------------------------ ###
### open new connection to write new CFT fracs ###
### ------------------------------------------ ###

nyear=read[2]-read[1]+1

# read and update header
cftin<- file(32_BAND_LANUSE_INPUT,"rb")
header.s <- readBin(cftin,raw(),size=1,n=7) # header string, to read use: rawToChar()
header.int <- readBin(cftin,integer(),size=4,n=7)
header.float <- readBin(cftin,double(),size=4,n=2)
close(cftin)

header.int[3]<- read[1]   # first year
header.int[4]<- nyear     # nyears
header.int[7]<- 64        # nbands

# write data into bin
con=file(64_BAND_LANDUSE_INPUT,"wb")
writeBin(as.raw(header.s),con,size=1)
writeBin(as.integer(header.int),con,size=4)
writeBin(as.double(header.float),con,size=4)

### ------------------------------------ ###
### year loop
### produce new cft fractions by year
### ------------------------------------ ###

cftin<- file(32_BAND_LANUSE_INPUT,"rb") # Mattis new data
seek(cftin,where=43+(ncell*nband*2*(read[1]-refyear)),origin="start")

irr_prev=array(0,c(ncell,16))
irrsystems_prev=array(0,c(ncell,3*16))

for (y in 1:nyear) {
  print(paste("year: ",read[1]+y-1))

  ### read land use input
  cft <- readBin(cftin,integer(),size=2,n=67420*32)

  cft<- cft*0.001 # scaling factor
  dim(cft)<- c(nband,ncell)
  cft<- t(cft)

  # clip cftfrac if > 1, reduce pastures or others
  for (i in 1:ncell) {
    if(sum(cft[i,])>1) {
      if(cft[i,14]> sum(cft[i,])-1) {
        cft[i,14]<- cft[i,14]-(sum(cft[i,])-1)
        next
      }
      if(cft[i,13]> sum(cft[i,])-1) {
        cft[i,13]<- cft[i,13]-(sum(cft[i,])-1)
        next
      }
      print("### Error: cftfrac reduction not succesfull, no enough others/pastures available ###")
    } # cellfrac > 1?
  }

  rain<- cft[,c(1:16)]
  irr<- cft[,c(17:32)]
  rm(cft)

  # assign irrigated tropical roots to rain-fed tropical roots, because they are generally not irrigated
  rain[which(irr[,7]>0),7] = rain[which(irr[,7]>0),7] + irr[which(irr[,7]>0),7]
  irr[which(irr[,7]>0),7] = 0

  irrarea<- irr*cellarea
  colnames(irrarea)<- c(1:16)
  rownames(irrarea)<- 1:ncell

  # final new table containing 3 irrigation system with 16 bands each
  irrsystems<-array(0,c(ncell,16*3))

  # before year 1900 put all to surface irrigation
  if (y <= 200) {
    irrsystems[,1:16]<- irr
  } else {
    if(y==201) {
      # rest in year 1900 to allow new sprikler and drip allocation
      irrsystems_prev=array(0,c(ncell,16*3))
      irr_prev=array(0,c(ncell,16))
    }
    ### country loop
    for (id in 1:length(c.names$Number))
    {
      cntrname=as.character(c.names$LPJ_Names[id])
      print(paste(" -------",cntrname," ------- "))
      cntrcells=which(lpj.countries==(id-1))
      share=as.vector(unlist(cntr_shares[cntr_shares$Country==cntrname,c(2,3,4)]))
      irrsystems[cntrcells,]<-0 # only needed during development, to not always run cntr loop from start
      
      if(sum(irrarea[cntrcells,])>0) 
      {
        # irr extent equal to last year?
        if(sum(irr[cntrcells,]!=irr_prev[cntrcells,])==0)
        {
          irrsystems[cntrcells,]<-irrsystems_prev[cntrcells,]
        }
        else
        {
          if(sum(share)==0) share=c(1,0,0)

          # new crop areas to be added 
          add_new=data.frame(cell=NULL,band=NULL) # cell + band
          for(c in 1:length(cntrcells)) for(b in 1:16) if(irr[cntrcells[c],b]>0 && irr_prev[cntrcells[c],b]==0) add_new=rbind(add_new,c(cntrcells[c],b))
          dripcells=NULL
          sprinkcells=NULL

          ### --------------- ###
          ### fill drip areas ###
          ### --------------- ###

          if(share[DRIP]>0) 
          {
            dripsystems=ifelse(irrsystems_prev[cntrcells,DRIPBANDS]>0,1,0) # cells and bands equipped with drip last year
            irrsystems[cntrcells,DRIPBANDS]<-irr[cntrcells,]*dripsystems # use last year's distribution

            target=sum(irrarea[cntrcells,])*share[DRIP]
            current_ext=sum(irrsystems[cntrcells,DRIPBANDS]*cellarea[cntrcells])
            driparea_def= target-current_ext # actual driparea deficit that needs to be closed this year >0 expansion, <0 reduction

            # current extent too small
            if(driparea_def>0)
            {
              dripsuit<- irrarea[cntrcells,which(drip_suit==TRUE)]
              dripsuit[irr_prev[cntrcells,which(drip_suit==TRUE)]>0]<- 0 # cell/bands that had zero cropfrac last year
              dripsuit<-matrix(dripsuit,nrow=length(cntrcells),dimnames=list(cntrcells,which(drip_suit==TRUE)))
              if(ifelse(length(unique(sort(dripsuit)))>1,driparea_def>unique(sort(dripsuit))[2],FALSE)) {
                
                # create table to sample from
                driptab=NULL
                for(b in 1:length(dimnames(dripsuit)[[2]])) 
                {
                  d=data.frame(area=dripsuit[,b],cell=as.numeric(dimnames(dripsuit)[[1]]),band=rep(as.numeric(dimnames(dripsuit)[[2]][b]),nrow(dripsuit)))
                  driptab=rbind(driptab,d[which(d$area>0),])
                }
                if(sum(driptab$area)<driparea_def) print(paste("WARNING: not enough suitable area to realize drip share in:",cntrname))
                
                # sample from potential drip areas until target value is reached, use best iteration out of 1000
                bestguess=driparea_def
                dripcells=NULL

                if(share[SPRINK]==0 && share[SURF]==0) 
                {
                  dripcells=driptab
                } 
                else 
                {
                  for(i in 1:100) 
                  {
                    dtab=driptab
                    accu=0
                    signed=NULL
                    
                    while(accu<=driparea_def && nrow(dtab)>0) 
                    {
                      row=sample(nrow(dtab),1)
                      if(accu+dtab[row,"area"]<driparea_def) 
                      {
                        accu=accu+dtab[row,"area"]
                        signed=rbind(signed,dtab[row,])
                      }
                      dtab=dtab[-row,]
                    } # while
                    res=driparea_def-accu
                    if(res<bestguess) 
                    {
                      bestguess=res
                      dripcells=signed
                    }
                    # if no cell was found because drip share too small
                    if(accu==0 && min(driptab$area)>driparea_def) 
                    {
                      dripcells=driptab[which(driptab$area==min(driptab$area)),]
                      print("single smallest cell assigned")
                      break
                    }
                    if(bestguess/driparea_def<1e-6) break
                  } # iterations
                  #print(paste("iterations:",i))
                  #print(paste("accuracy:",1-(bestguess/driparea)))
                  rm(bestguess,res,signed,dtab)
                } # end of if sprink and surf shares are not NULL
                
                ## write selected drip areas into final table
                for (i in 1:nrow(dripcells)) 
                {
                  irrsystems[dripcells[i,"cell"],32+dripcells[i,"band"]]<- irr[dripcells[i,"cell"],dripcells[i,"band"]]
                  if(nrow(add_new)>0) {
                    if(dripcells[i,"cell"]%in%add_new[,1] && dripcells[i,"band"]%in%add_new[,2]) add_new=add_new[-c(which(add_new[,1]==dripcells[i,"cell"] & add_new[,2]==dripcells[i,"band"])),]
                  }
                }
                rm(driptab)
              } # driparea_def > min cell frac
              rm(dripsuit)
            } # current extent too small

            ### ------------------ ###

            # current drip extent too large / deficit larger than min. cell fraction?
            if(driparea_def<0 && ifelse(length(unique(sort(irrsystems[cntrcells,DRIPBANDS[which(drip_suit==TRUE)]]*cellarea[cntrcells])))>1,
              abs(driparea_def)>unique(sort(irrsystems[cntrcells,DRIPBANDS[which(drip_suit==TRUE)]]*cellarea[cntrcells]))[2],FALSE))
            {
              #print("current extent too large: reducing...")
              dripsuit<- irrsystems[cntrcells,DRIPBANDS[which(drip_suit==TRUE)]]*cellarea[cntrcells]
              dripsuit<-matrix(dripsuit,nrow=length(cntrcells),dimnames=list(cntrcells,which(drip_suit==TRUE)))
              
              # create table to sample from
              driptab=NULL
              for(b in 1:length(dimnames(dripsuit)[[2]])) 
              {
                d=data.frame(area=dripsuit[,b],cell=as.numeric(dimnames(dripsuit)[[1]]),band=rep(as.numeric(dimnames(dripsuit)[[2]][b]),nrow(dripsuit)))
                driptab=rbind(driptab,d[which(d$area>0),])
              }
              if(sum(driptab$area)<abs(driparea_def)) print(paste("WARNING: not enough suitable area to realize drip share in:",cntrname))
              
              # sample from potential drip areas until target value is reached, use best iteration out of 1000
              bestguess=abs(driparea_def)

              if(share[SPRINK]==0 && share[SURF]==0) 
              {
                dripcells=driptab
              } else {
                for(i in 1:100) 
                {
                  dtab=driptab
                  accu=0
                  signed=NULL
                  
                  while(accu<=abs(driparea_def) && nrow(dtab)>0) 
                  {
                    row=sample(nrow(dtab),1)
                    if(accu+dtab[row,"area"]<abs(driparea_def)) 
                    {
                      accu=accu+dtab[row,"area"]
                      signed=rbind(signed,dtab[row,])
                    }
                    dtab=dtab[-row,]
                  } # while
                  res=abs(driparea_def)-accu
                  if(res<bestguess) 
                  {
                    bestguess=res
                    dripcells=signed
                  }
                  # if no cell was found because drip share too small
                  if(accu==0 && min(driptab$area)>abs(driparea_def)) 
                  {
                    dripcells=driptab[which(driptab$area==min(driptab$area)),]
                    print("single smallest cell assigned")
                    break
                  }
                  if(bestguess/abs(driparea_def)<1e-4) break
                } # iterations
                #print(paste("iterations:",i))
                #print(paste("accuracy:",1-(bestguess/driparea)))
                rm(bestguess,res,signed,dtab)
              } # end of if sprink and surf shares are not NULL
              
              ## write selected drip areas into final table
              for (i in 1:nrow(dripcells)) 
              {
                irrsystems[dripcells[i,"cell"],32+dripcells[i,"band"]]<- 0
                add_new=rbind(add_new,c(dripcells[i,"cell"],dripcells[i,"band"])) # add cell/bands that were freed from drip
              }
              rm(dripsuit,driptab)

            } # current extent too large

            rm(target,current_ext,dripsystems)

          } # drip allocation
          
          ### ----------------- ###
          ### fill sprink areas ###
          ### ----------------- ###
          if(share[SPRINK]>0)
          {
            #print("--- sprinkler ---")
            sprinksystems=ifelse(irrsystems_prev[cntrcells,SPRINKBANDS]>0,1,0) # cells and bands equipped with drip last year
            irrsystems[cntrcells,SPRINKBANDS]<-irr[cntrcells,]*sprinksystems # use last year's distribution

            target=sum(irrarea[cntrcells,])*share[SPRINK]
            current_ext=sum(irrsystems[cntrcells,SPRINKBANDS]*cellarea[cntrcells])
            sprinkarea_def= target-current_ext # actual driparea deficit that needs to be closed this year >0 expansion, <0 reduction

            # current extent too small  
            if(sprinkarea_def>0)
            {
              sprinksuit<- irrarea[cntrcells,which(sprink_suit==TRUE)]
              sprinksuit[irr_prev[cntrcells,which(sprink_suit==TRUE)]>0]<- 0 # cell/bands that had zero cropfrac last year
              sprinksuit<-matrix(sprinksuit,nrow=length(cntrcells),dimnames=list(cntrcells,which(sprink_suit==TRUE))) 

              if(nrow(data.frame(dripcells))>0 && driparea_def>0) # delete areas already assigned to drip
                for (i in 1:nrow(dripcells)) sprinksuit[dimnames(sprinksuit)[[1]]==dripcells[i,"cell"],dimnames(sprinksuit)[[2]]==dripcells[i,"band"]]<- 0
              if(nrow(data.frame(dripcells))>0 && driparea_def<0)
                for (i in 1:nrow(dripcells)) sprinksuit[dimnames(sprinksuit)[[1]]==dripcells[i,"cell"],dimnames(sprinksuit)[[2]]==dripcells[i,"band"]]<- irrarea[dripcells[i,"cell"],dripcells[i,"band"]]

              if(ifelse(length(unique(sort(sprinksuit)))>1,sprinkarea_def>unique(sort(sprinksuit))[2],FALSE)) {
              
                # create table to sample from
                sprinktab=NULL
                for(b in 1:length(dimnames(sprinksuit)[[2]])) 
                {
                  d=data.frame(area=sprinksuit[,b],cell=as.numeric(dimnames(sprinksuit)[[1]]),band=rep(as.numeric(dimnames(sprinksuit)[[2]][b]),nrow(sprinksuit)))
                  sprinktab=rbind(sprinktab,d[which(d$area>0),])
                }
                #if(sum(sprinktab$area)<sprinkarea_def) print(paste("WARNING: not enough suitable area to realize sprinkler share in:",cntrname))
                
                # sample from potential sprink areas until target value is reached, use best iteration out of 1000
                bestguess=sprinkarea_def
                
                if(share[SURF]>0) 
                {
                  for(i in 1:100) 
                  {
                    dtab=sprinktab
                    accu=0
                    signed=NULL
                    
                    while(accu<=sprinkarea_def && nrow(dtab)>0) 
                    {
                      row=sample(nrow(dtab),1)
                      if(accu+dtab[row,"area"]<=sprinkarea_def) 
                      {
                        accu=accu+dtab[row,"area"]
                        signed=rbind(signed,dtab[row,])
                      }
                      dtab=dtab[-row,]
                    } # while
                    res=sprinkarea_def-accu
                    if(res<bestguess) 
                    {
                      bestguess=res
                      sprinkcells=signed
                    }
                    # if no cell was found because sprink share too small
                    if(accu==0 && min(sprinktab$area)>sprinkarea_def) 
                    {
                      sprinkcells=sprinktab[which(sprinktab$area==min(sprinktab$area)),]
                      print("single smallest cell assigned")
                      break
                    }
                    if(bestguess/sprinkarea_def<1e-5) break
                  } # iterations
                  #print(paste("iterations:",i))
                  #print(paste("accuracy:",1-(bestguess/sprinkarea_def)))
                  rm(bestguess,res,signed,dtab)
                } else {
                  sprinkcells=sprinktab # if no surf share, put all remaining to sprink
                }
            
                ## write selected sprink areas into final table
                for (i in 1:nrow(sprinkcells)) 
                {
                  irrsystems[sprinkcells[i,"cell"],16+sprinkcells[i,"band"]]<- irr[sprinkcells[i,"cell"],sprinkcells[i,"band"]]
                  if(nrow(add_new)>0) {
                    if(sprinkcells[i,"cell"]%in%add_new[,1] && sprinkcells[i,"band"]%in%add_new[,2]) add_new=add_new[-c(which(add_new[,1]==sprinkcells[i,"cell"] & add_new[,2]==sprinkcells[i,"band"])),]
                  }       
                }
                rm(sprinktab)
              } # sprinkarea_def > min cell frac?
              rm(sprinksuit)

            } # sprinkarea_def > 0

            ### ------------------ ###

            # current drip extent too large
            if(sprinkarea_def<0 && ifelse(length(unique(sort(irrsystems[cntrcells,SPRINKBANDS[which(sprink_suit==TRUE)]]*cellarea[cntrcells])))>1,
              abs(sprinkarea_def)>unique(sort(irrsystems[cntrcells,SPRINKBANDS[which(sprink_suit==TRUE)]]*cellarea[cntrcells]))[2],FALSE))
            {
              #print("current extent too large: reducing...")
              sprinksuit<- irrsystems[cntrcells,SPRINKBANDS[which(sprink_suit==TRUE)]]*cellarea[cntrcells]
              sprinksuit<-matrix(sprinksuit,nrow=length(cntrcells),dimnames=list(cntrcells,which(sprink_suit==TRUE))) 
              
              # create table to sample from
              sprinktab=NULL
              for(b in 1:length(dimnames(sprinksuit)[[2]])) 
              {
                d=data.frame(area=sprinksuit[,b],cell=as.numeric(dimnames(sprinksuit)[[1]]),band=rep(as.numeric(dimnames(sprinksuit)[[2]][b]),nrow(sprinksuit)))
                sprinktab=rbind(sprinktab,d[which(d$area>0),])
              }
              #if(sum(sprinktab$area)<abs(sprinkarea_def)) print(paste("WARNING: not enough suitable area to realize sprinkler share in:",cntrname))
              
              # sample from potential sprink areas until target value is reached, use best iteration out of 1000
              bestguess=abs(sprinkarea_def)
              sprinkcells=NULL
              
              if(share[SURF]>0) 
              {
                for(i in 1:100) 
                {
                  dtab=sprinktab
                  accu=0
                  signed=NULL
                  
                  while(accu<=abs(sprinkarea_def) && nrow(dtab)>0) 
                  {
                    row=sample(nrow(dtab),1)
                    if(accu+dtab[row,"area"]<=abs(sprinkarea_def)) 
                    {
                      accu=accu+dtab[row,"area"]
                      signed=rbind(signed,dtab[row,])
                    }
                    dtab=dtab[-row,]
                  } # while
                  res=abs(sprinkarea_def)-accu
                  if(res<bestguess) 
                  {
                    bestguess=res
                    sprinkcells=signed
                  }
                  # if no cell was found because sprink share too small
                  if(accu==0 && min(sprinktab$area)>abs(sprinkarea_def)) 
                  {
                    sprinkcells=sprinktab[which(sprinktab$area==min(sprinktab$area)),]
                    print("single smallest cell assigned")
                    break
                  }
                  if(bestguess/abs(sprinkarea_def)<1e-5) break
                } # iterations
                #print(paste("iterations:",i))
                #print(paste("accuracy:",1-(bestguess/sprinkarea_def)))
                rm(bestguess,res,signed,dtab)
              } else {
                sprinkcells=sprinktab # if no surf share, put all remaining to sprink
              }
          
              ## write selected sprink areas into final table
              for (i in 1:nrow(sprinkcells)) 
              {
                irrsystems[sprinkcells[i,"cell"],16+sprinkcells[i,"band"]]<- 0
                add_new=rbind(add_new,c(sprinkcells[i,"cell"],sprinkcells[i,"band"])) # add cell/bands that were freed from drip
              }
              rm(sprinksuit,sprinktab)

            } # sprinkarea_def<0

            rm(target,current_ext,sprinkarea_def,sprinksystems)
            
          } # sprink allocation

          ### --------------- ###
          ### fill surf areas ###
          ### --------------- ###
          
          if(share[SURF]>0) {
            #print("--- surface ---")
            surfsystems=ifelse(irrsystems_prev[cntrcells,SURFBANDS]>0,1,0) # cells and bands equipped with drip last year
            irrsystems[cntrcells,SURFBANDS]<-irr[cntrcells,]*surfsystems # use last year's distribution

            target=sum(irrarea[cntrcells,])*share[SURF]
            current_ext=sum(irrsystems[cntrcells,SURFBANDS]*cellarea[cntrcells])
            surfarea_def= target-current_ext # actual driparea deficit that needs to be closed this year >0 expansion, <0 reduction

            if(nrow(add_new)>0) {
              for (i in 1:nrow(add_new)) {
                irrsystems[add_new[i,1],add_new[i,2]]<- irr[add_new[i,1],add_new[i,2]]
                add_new[i,]=c(0,0)
              }
              add_new=add_new[-which(add_new[,1]==0),]
            }

            rm(target,current_ext,surfarea_def,surfsystems)
          } # surface allocation

          # assign remaining cell/bands, if possible first to surface, then to sprinkler, last to drip
          if(nrow(add_new)>0) {
            for (i in 1:nrow(add_new)) {
              if(share[SURF]>0) irrsystems[add_new[i,1],add_new[i,2]]<- irr[add_new[i,1],add_new[i,2]]
              if(share[SURF]==0 && share[SPRINK]>0) irrsystems[add_new[i,1],16+add_new[i,2]]<- irr[add_new[i,1],add_new[i,2]]
              if(share[SURF]==0 && share[SPRINK]==0) irrsystems[add_new[i,1],32+add_new[i,2]]<- irr[add_new[i,1],add_new[i,2]]
              add_new[i,]=c(0,0)
            }
            add_new=add_new[-c(add_new[1,]==0),]
          }

          ## test if shares in new irrigsystems are identical
          if(sum(apply(matrix(irrsystems[cntrcells,],nrow=length(cntrcells)),1,sum)==apply(matrix(irr[cntrcells,],nrow=length(cntrcells)),1,sum))!=length(cntrcells)) {
            print(paste("ERROR: cftfrac is not kept constant:",cntrname))
            break
          }

          # test if country shares are actually met  
          #su=sum(apply(matrix(irrsystems[cntrcells,1:16],nrow=length(cntrcells)),1,sum)*cellarea[cntrcells])
          #sp=sum(apply(matrix(irrsystems[cntrcells,17:32],nrow=length(cntrcells)),1,sum)*cellarea[cntrcells])
          #dr=sum(apply(matrix(irrsystems[cntrcells,33:48],nrow=length(cntrcells)),1,sum)*cellarea[cntrcells])
          #share_test=round(c(su/(su+sp+dr),sp/(su+sp+dr),dr/(su+sp+dr)),d=3)
          #print(paste("FAO  share:",share))
          #print(paste("test share:",share_test))
          #if(max(abs(share-share_test))>0.001) print(paste("WARNING: Country shares are not precisely met in",cntrname))
          #rm(su,sp,dr,share_test)

        } # irr extent changed this year
      } # enf if irrarea > 0
    } # end of country loop
  } # after year 1900?

  # test if shares in new irrigsystems are identical
  if(sum(apply(irrsystems,1,sum)==apply(irr,1,sum))!=ncell) 
  {
    print(paste("ERROR: shares in irrsystems not equal to cftfrac globally"))
    #break
  }

  ### ------------------------------------------- ###
  ### store to see if next year changes necessary ###
  ### ------------------------------------------- ###

  irrsystems_prev=irrsystems
  irr_prev=irr

  ### -------------------------- ###
  ### write into open connection ###
  ### -------------------------- ###

  # create data vector
  cftfrac_new=round(cbind(rain,irrsystems)*1000) # produce integers
  cftbin=c(t(cftfrac_new)) # create a vector with dims: nbands,ncell

  # write data into bin
  writeBin(as.integer(cftbin),con,size=2)
  rm(cftbin,rain,irrsystems,cftfrac_new,irr,share,irrarea)

} # year loop

close(cftin)
close(con)

### ------- ###
print("Ready!")
### ------- ###
