rm(list=ls(all=TRUE))
gc()

######################################
# libraries ####
######################################
require(fields)
require(maps)
require(ncdf4)

#######################################
# global settings ####
#######################################
NODATA <- 1e20
plot.maps <- T
ncell <- 67420
input.path <- "/data/biosx/LPJ/input_longheader/" 
#out.path <- "/iplex/01/2011/isimip-lpj/yields/GGCMI2_outputs/"
out.path <- "/p/projects/macmit/data/GGCMI/CTWN/agmerra_fullharm/"

model <- c("lpjml")
gcms <- c("WATCH","WFDEI_CRU","WFDEI_GPCC","AgMERRA","AgCFSR","Princeton","GRASP","ERAI","CFSR","PGFv2","GSWP3","PGFv21")
climate <- c("watch","wfdei.cru","wfdei.gpcc","agmerra","agcfsr","princeton","grasp","erai","cfsr","pgfv2","gswp3","pgfv2")
clim.scenario <- "hist"
harms <- c("default","fullharm")
harmnames <- c("default","harmnon")
irrigs <- c("noirr","firr")
crops <- c("whe","ric","mai","mil","pea","sgb","cas","sun","soy","nut","rap","sug","mgr")
cropf <- c("wheat","rice","maize","millet","others","sugar_beet","others","others","soy","others",
           "rapeseed","sugarcane","managed_grass")
variables <- c("yield","pirrww","biom","aet","plant-day","maty-day","gsprcp","gsrsds","sumt")
timestep <- "annual"

# per climate
start.year <- c(1958,1979,1979,1980,1980,1948,1961,1979,1980,1901,1901,1901)
first.year <- c(1901,1979,1979,1980,1980,1948,1961,1979,1980,1901,1901,1901)
end.year <- c(2001,2012,2010,2010,2010,2008,2010,2010,2010,2012,2010,2012)

# per variable
names <- c("yield","pirrww","biom","aet","plant-day","maty-day","gsprcp","gsrsds","sumt")
units <- c("t ha-1 yr-1","mm yr-1","t ha-1 yr-1","mm yr-1","day of year","days from planting",
           "mm yr-1","w m-2 yr-1","deg C-days yr-1")
longnames <- c("Crop yields","Applied irrigation water","Total Above ground biomass yield",
               "Actual growing season evapotranspiration","Actual planting date","Days from planting to maturity",
               "Accumulated precipitation, planting to harvest","Growing season incoming solar",
               "Sum of daily mean temps, planting to harvest")



#######################################
# functions ####
#######################################
read.LPJmL.grid <- function(filename,npix,header,swap=F){
  ff <- file(filename,"rb")
  seek(ff,where=header,origin="start")
  if(swap){
    x <- readBin(ff,integer(),size=2,n=npix*2,endian="swap")/100
  } else {
    x <- readBin(ff,integer(),size=2,n=npix*2)/100
  }
  lon <- x[(1:npix)*2-1]
  lat <- x[(1:npix)*2]
  # ilon/ilat are row and column numbers, starting at 179.75W and 89.75S as 1/1
  ilon <- as.integer((lon+180)/0.5 + 1.01)
  ilat <- as.integer((lat+90)/0.5 + 1.01)
  area <- (111e3*0.5)*(111e3*0.5)*cos(lat/180*pi)/10000
  close(ff)
  data.frame(lon,lat,ilon,ilat,area)
}

read.LPJmL.array.mean <- function(filename,fyear,syear,eyear,nbands,crops,npix){
  ff <- file(filename,"rb")
  data <- array(0,dim=c(npix,length(crops)))
  for(y in syear:eyear){
    for(cr in crops){
      seek(ff,where=((y-fyear)*nbands+(cr-1))*4*npix,origin="start")
      data[,cr] <- data[,cr] + readBin(ff,double(),size=4,n=npix)/(eyear-syear+1)
    }
  }
  close(ff)
  data
}

read.LPJmL.array <- function(filename,fyear,syear,eyear,nbands,crop,npix){
  cat("reading",filename,"\n")
  ff <- file(filename,"rb")
  data <- array(NA,dim=c(npix,(eyear-syear+1)))
  for(y in syear:eyear){
    seek(ff,where=((y-fyear)*nbands+(crop-1))*4*npix,origin="start")
    data[,(y-syear+1)] <- readBin(ff,double(),size=4,n=npix)
  }
  close(ff)
  data
}

read.LPJmL.array.int <- function(filename,fyear,syear,eyear,nbands,crop,npix){
  ff <- file(filename,"rb")
  data <- array(NA,dim=c(npix,(eyear-syear+1)))
  for(y in syear:eyear){
    seek(ff,where=((y-fyear)*nbands+(crop-1))*2*npix,origin="start")
    data[,(y-syear+1)] <- readBin(ff,integer(),size=2,n=npix)
  }
  close(ff)
  data
}

read.landuse <- function(filename,years,yeare){
  ff <- file(filename,"rb")
  seek(ff,where=43+32*ncell*(yeare-1700)*2,origin="start")
  x <- matrix(readBin(ff,integer(),n=32*ncell,size=2)/1000,nrow=ncell,byrow=T)
  for(y in years:(yeare-1)){
    seek(ff,where=43+32*ncell*(yeare-1700)*2,origin="start")
    x2 <- matrix(readBin(ff,integer(),n=32*ncell,size=2)/1000,nrow=ncell,byrow=T)
    x[x>x2] <- x2[x>x2]    
  }
  close(ff)
  x
}

replace_previous <- function(vec,vec2,y,yr){
  vec[yr] <- vec[y]
  vec[y] <- vec2[y]
  vec
}

discard_first <- function(data,select){
  data[select,c(1:(dim(data)[2]-1))] <- data[select,c(2:dim(data)[2])]
  data[select,dim(data)[2]] <- 0
  data
}



#######################################
# main ####
#######################################

grid <- read.LPJmL.grid(paste(input.path,"grid.bin",sep=""),ncell,43,swap=F)
lu <- read.landuse(paste(input.path,"cft1700_2005_bioenergy_sc.bin",sep=""),2000,2000)

parallel<-TRUE
parallel <- F

if(parallel==T) {
	library(Rmpi)  # R implementation of MPI interface
	library(doMPI) # interface for foreach construct to run in MPI parallel mode
	clu <- startMPIcluster(verbose=F) # start cluster (link R instances together)
	num.cluster <- clusterSize(clu)
	if (num.cluster > 1) {
		# we are using more than 1 CPU, so really run in parallel mode
		registerDoMPI(clu) # tells foreach to use MPI parallel mode
		print(paste("Running in parallel mode on",num.cluster,"worker nodes."))
	} else {
		registerDoSEQ() # tells foreach to use sequential mode
		print("Running in sequential mode.")
	}
} else {
	library(foreach)
	registerDoSEQ() # tells foreach to use sequential mode
	print("Running in sequential mode.")
} 

#for(cl in 1:length(climate)){
for(cl in c(4)){
  #for(ha in 1:length(harms)){
  for(ha in 2){
    for(ir in 1:length(irrigs)){
      for(cr in 1:length(crops)){
      #parloop <- foreach(cr=c(1:length(crops)), .errorhandling='pass', .verbose=TRUE, .export='message') %dopar% {
        cat("doing",crops[cr],"\n")
        #txt <- paste(txt,"\n","Running on worker", mpi.comm.rank(0),"\n")
        # read in all data here to have double harvest fix in one go
        data.path <- paste(out.path,sep="")
        fn <- paste(data.path,"/",model,"_",climate[cl],"_",clim.scenario,"_",harmnames[ha],"_",crops[cr],"_",irrigs[ir],"_",start.year[cl],"_",end.year[cl],".Rdata",sep="")
        # deleting all Rdata, which contains the outdated sh1t.
        if(file.exists(fn)) unlink(fn)
        cat("doing",fn,"\n")
        if(!file.exists(fn)){
          yld <- read.LPJmL.array(paste(data.path,"/pft_harvest.bin",sep=""),first.year[cl],start.year[cl],end.year[cl],32,if(ir>1) cr+16 else cr,ncell)/0.45/100
          yld2 <- read.LPJmL.array(paste(data.path,"/pft_harvest.bin",sep=""),first.year[cl],start.year[cl],end.year[cl],32,if(ir>1) cr+16 else cr,ncell)/0.45/100
          yld2[] <- 0
          # if(ir>1){
          #   piw <- read.LPJmL.array(paste(data.path,"/pft_airrig.bin",sep=""),first.year[cl],start.year[cl],end.year[cl],32,if(ir>1) cr+16 else cr,ncell)
          #   piw2 <- read.LPJmL.array(paste(data.path,"/pft_airrig2.bin",sep=""),first.year[cl],start.year[cl],end.year[cl],32,if(ir>1) cr+16 else cr,ncell)
          # }
          # bim <- read.LPJmL.array(paste(data.path,"/cft_abovegbm.bin",sep=""),first.year[cl],start.year[cl],end.year[cl],28,if(ir>1) cr+14 else cr,ncell)/0.45/100
          # bim2 <- read.LPJmL.array(paste(data.path,"/cft_abovegbm2.bin",sep=""),first.year[cl],start.year[cl],end.year[cl],28,if(ir>1) cr+14 else cr,ncell)/0.45/100
          # aet <- read.LPJmL.array(paste(data.path,"/cft_evap.bin",sep=""),first.year[cl],start.year[cl],end.year[cl],32,if(ir>1) cr+16 else cr,ncell)
          # aet <- aet + read.LPJmL.array(paste(data.path,"/cft_transp.bin",sep=""),first.year[cl],start.year[cl],end.year[cl],32,if(ir>1) cr+16 else cr,ncell)
          # aet <- aet + read.LPJmL.array(paste(data.path,"/cft_interc.bin",sep=""),first.year[cl],start.year[cl],end.year[cl],32,if(ir>1) cr+16 else cr,ncell)
          # aet2 <- read.LPJmL.array(paste(data.path,"/cft_evap2.bin",sep=""),first.year[cl],start.year[cl],end.year[cl],32,if(ir>1) cr+16 else cr,ncell)
          # aet2 <- aet2 + read.LPJmL.array(paste(data.path,"/cft_transp2.bin",sep=""),first.year[cl],start.year[cl],end.year[cl],32,if(ir>1) cr+16 else cr,ncell)
          # aet2 <- aet2 + read.LPJmL.array(paste(data.path,"/cft_interc2.bin",sep=""),first.year[cl],start.year[cl],end.year[cl],32,if(ir>1) cr+16 else cr,ncell)
          # if(cr<length(crops)){
          #   pdy <- read.LPJmL.array.int(paste(data.path,"/sdate.bin",sep=""),first.year[cl],start.year[cl],end.year[cl],24,if(ir>1) cr+12 else cr,ncell)
          #   # sdate never happens twice with prescribed sdates.
          #   #pdy2 <- read.LPJmL.array.int(paste(data.path,"/sdate2.bin",sep=""),start.year[cl],start.year[cl],end.year[cl],24,if(ir>1) cr+12 else cr,ncell)
          # } else pdy <- array(0,dim=dim(yld))
          # mdy <- read.LPJmL.array(paste(data.path,"/growing_period.bin",sep=""),first.year[cl],start.year[cl],end.year[cl],28,if(ir>1) cr+14 else cr,ncell)
          # mdy2 <- read.LPJmL.array(paste(data.path,"/growing_period2.bin",sep=""),first.year[cl],start.year[cl],end.year[cl],28,if(ir>1) cr+14 else cr,ncell)
          # gsp <- read.LPJmL.array(paste(data.path,"/cft_prec.bin",sep=""),first.year[cl],start.year[cl],end.year[cl],28,if(ir>1) cr+14 else cr,ncell)
          # gsp2 <- read.LPJmL.array(paste(data.path,"/cft_prec2.bin",sep=""),first.year[cl],start.year[cl],end.year[cl],28,if(ir>1) cr+14 else cr,ncell)
          # gsr <- read.LPJmL.array(paste(data.path,"/cft_srad.bin",sep=""),first.year[cl],start.year[cl],end.year[cl],28,if(ir>1) cr+14 else cr,ncell)
          # gsr2 <- read.LPJmL.array(paste(data.path,"/cft_srad2.bin",sep=""),first.year[cl],start.year[cl],end.year[cl],28,if(ir>1) cr+14 else cr,ncell)
          # gst <- read.LPJmL.array(paste(data.path,"/cft_temp.bin",sep=""),first.year[cl],start.year[cl],end.year[cl],28,if(ir>1) cr+14 else cr,ncell)
          # gst2 <- read.LPJmL.array(paste(data.path,"/cft_temp2.bin",sep=""),first.year[cl],start.year[cl],end.year[cl],28,if(ir>1) cr+14 else cr,ncell)
          # if(cr<length(crops)){
          #   syr <- read.LPJmL.array.int(paste(data.path,"/syear.bin",sep=""),first.year[cl],start.year[cl],end.year[cl],24,if(ir>1) cr+12 else cr,ncell)
          #   syr2 <- read.LPJmL.array.int(paste(data.path,"/syear.bin",sep=""),first.year[cl],start.year[cl],end.year[cl],24,if(ir>1) cr+12 else cr,ncell)
          #   syr2[] <- 0
          #   # discarding first years' values if syear is before first year ####
          #   discardold <- which((pdy[,1]+mdy[,1])>365)
          #   discard <- which(syr[,1]<start.year[cl])
          #   discard2 <- which(syr2[,1]<start.year[cl])
          #   cat("discarding",length(discard2), length(discard),"\n")
          #   cat("discarding first year values in",length(discard),"cases\n")
          #   yld <- discard_first(yld,discard)
          #   yld2 <- discard_first(yld2,discard2)
          #   # if(ir>1){
          #   #   piw <- discard_first(piw,discard)
          #   #   piw2 <- discard_first(piw2,discard2)
          #   # }
          #   # bim <- discard_first(bim,discard)
          #   # bim2 <- discard_first(bim2,discard2)
          #   # aet <- discard_first(aet,discard)
          #   # aet2 <- discard_first(aet2,discard2)
          #   # pdy <- discard_first(pdy,discard)
          #   # #pdy2 <- discard_first(pdy2,discard)
          #   # mdy <- discard_first(mdy,discard)
          #   # mdy2 <- discard_first(mdy2,discard2)
          #   # gsp <- discard_first(gsp,discard)
          #   # gsp2 <- discard_first(gsp2,discard2)
          #   # gsr <- discard_first(gsr,discard)
          #   # gsr2 <- discard_first(gsr2,discard2)
          #   # gst <- discard_first(gst,discard)
          #   # gst2 <- discard_first(gst2,discard2)
          #   # syr <- discard_first(syr,discard)
          #   # syr2 <- discard_first(syr2,discard2)
          # }
          # 
          # # fixing double harvests ####
          # testn <- which(rowSums(yld2)>0)
          # # just double checking
          # testn2 <- which(rowSums(aet2)>0)
          # #if(length(testn)>0 & length(testn2)>0) if(testn!=testn2) cat("WARNING! different set of double harvests in YLD2\n",testn,"\nand AET\n",testn2,"\n")
          # if(length(testn)>0 & length(testn2)>0) if(length(which(testn%in%testn2))!=length(testn)) txt <- paste(txt,"WARNING! different set of double harvests in YLD2\n",paste(testn,collapse=" "),"\nand AET\n",paste(testn2,collapse=" "),"\n")
          # rm(testn2)
          # for(nfix in testn){
          #   # extracting vectors of pixels with double harvests
          #   b.yld <- v.yld <- yld[nfix,]
          #   b.yld2 <- v.yld2 <- yld2[nfix,]
          #   # if(ir>1){
          #   #   b.piw <- v.piw <- piw[nfix,]
          #   #   b.piw2 <- v.piw2 <- piw2[nfix,]
          #   # }
          #   # b.bim <- v.bim <- bim[nfix,]
          #   # b.bim2 <- v.bim2 <- bim2[nfix,]
          #   # b.aet <- v.aet <- aet[nfix,]
          #   # b.aet2 <- v.aet2 <- aet2[nfix,]
          #   # #v.pdy <- pdy[nfix,]
          #   # #v.pdy2 <- pdy2[nfix,]
          #   # b.mdy <- v.mdy <- mdy[nfix,]
          #   # b.mdy2 <- v.mdy2 <- mdy2[nfix,]
          #   # b.gsp <-v.gsp <- gsp[nfix,]
          #   # b.gsp2 <- v.gsp2 <- gsp2[nfix,]
          #   # b.gsr <- v.gsr <- gsr[nfix,]
          #   # b.gsr2 <- v.gsr2 <- gsr2[nfix,]
          #   # b.gst <- v.gst <- gst[nfix,]
          #   # b.gst2 <- v.gst2 <- gst2[nfix,]
          #   if(cr==length(crops)) {
          #     index <- c(1:(end.year[cl]-start.year[cl])+1)
          #   } else {
          #     index <- syr[nfix,]-start.year[cl]+1
          #     index2 <- syr2[nfix,]-start.year[cl]+1
          #     index[index<0] <- 0
          #     index2[index2<0] <- 0
          #   }
          #   yearswithsowing <- array(0,length(index))
          #   
          #   # fixing case by case
          #   # setting first harvests to right year
          #   for(yf in 1:length(index)){
          #     if(index[yf]!=0){
          #       ind <- index[yf]
          #       v.yld[ind] <- b.yld[yf]
          #       # if(ir>1) v.piw[index] <- b.piw[yf]
          #       # v.bim[ind] <- b.bim[yf]
          #       # v.aet[ind] <- b.aet[yf]
          #       # v.mdy[ind] <- b.mdy[yf]
          #       # v.gsp[ind] <- b.gsp[yf]
          #       # v.gsr[ind] <- b.gsr[yf]
          #       # v.gst[ind] <- b.gst[yf]
          #       yearswithsowing[ind] <- 1
          #     }
          #     if(index2[yf]!=0){
          #       ind <- index2[yf]
          #       v.yld[ind] <- b.yld2[yf]
          #       v.yld2[yf] <- 0
          #       # if(ir>1) {
          #       #   v.piw[ind] <- b.piw2[yf]
          #       #   v.piw2[yf] <- 0
          #       # }
          #       # v.bim[ind] <- b.bim2[yf]
          #       # v.bim2[yf] <- 0
          #       # v.aet[ind] <- b.aet2[yf]
          #       # v.aet2[yf] <- 0
          #       # v.mdy[ind] <- b.mdy2[yf]
          #       # v.mdy2[yf] <- 0
          #       # v.gsp[ind] <- b.gsp2[yf]
          #       # v.gsp2[yf] <- 0
          #       # v.gsr[ind] <- b.gsr2[yf]
          #       # v.gsr2[yf] <- 0
          #       # v.gst[ind] <- b.gst2[yf]
          #       # v.gst2[yf] <- 0
          #       yearswithsowing[ind] <- 1
          #     }
          #   }
          #   
          #   
          #   
          #   # testing if all cases have been fixed ####
          #   
          #   # yfix2 <- which(v.yld2>0)
          #   # if(length(yfix2)>0)
          #   #   if(yfix2[1]!=length(v.yld2)) txt <- paste(txt,"WARNING, still double harvests left:",paste(yfix2,collapse=" "),"\n")
          #   # yfix2 <- which(yearswithsowing==0)
          #   # if(length(yfix2)>0) 
          #   #   if(!(length(yfix2)==1 & yearswithsowing[length(yearswithsowing)]==0))txt <- paste(txt,"WARNING, still years without left:",paste(yfix2,collapse=" "),"\n")
          #   # if(length(yfix2)>1){
          #   #   txt <- paste(txt,paste(nfix,collapse=" "),"\n",paste(syr[nfix,],collapse=" "),"\n",paste(syr2[nfix,],collapse=" "),"\n",paste(yearswithsowing,collapse=" "),"\n",paste(v.yld,collapse=" "),"\n",paste(v.yld2,collapse=" "),"\n")
          #   # }
          #   
          #   # feeding back vectors with corrected double harvests
          #   # v.yld -> yld[nfix,]
          #   # v.yld2 -> yld2[nfix,]
          #   # if(ir>1){
          #   #   v.piw -> piw[nfix,]
          #   #   v.piw2 -> piw2[nfix,]
          #   # }
          #   # v.bim -> bim[nfix,]
          #   # v.bim2 -> bim2[nfix,]
          #   # v.aet -> aet[nfix,]
          #   # v.aet2 -> aet2[nfix,]
          #   # #v.pdy -> pdy[nfix,]
          #   # #v.pdy2 -> pdy2[nfix,]
          #   # v.mdy -> mdy[nfix,]
          #   # v.mdy2 -> mdy2[nfix,]
          #   # v.gsp -> gsp[nfix,]
          #   # v.gsp2 -> gsp2[nfix,]
          #   # v.gsr -> gsr[nfix,]
          #   # v.gsr2 -> gsr2[nfix,]
          #   # v.gst -> gst[nfix,]
          #   # v.gst2 -> gst2[nfix,]
          # }
          # save all corrected outputs to Rdata file
          # if(ir==1) piw <- NA # placeholder for non-existant PIW in rainfed cases
          # save(yld,piw,bim,aet,pdy,mdy,gsp,gsr,gst,file=fn,compress=T)
        } else {
          # if binary data has been processed already, read from Rdata file
          load(fn, .GlobalEnv)
        }
        #for(va in 1:length(variables)){
        for(va in 1){
          if(!(va==2 & ir==1)){
            dir.create(file.path(data.path,"ncdf"), showWarnings = FALSE)
            #dir.create(file.path(data.path,"ncdf",cropf[cr]), showWarnings =F)
            opath <- file.path(data.path,"ncdf","")
            # [model]_[climate]_[clim.scenario]_[sim.scenario]_[variable]_[crop]_[timestep]_[start-year]_[end-year].nc4 
            fn <- paste(opath,model,"_",climate[cl],"_",clim.scenario,"_",harmnames[ha],"_",irrigs[ir],"_",variables[va],"_",crops[cr],"_",timestep,"_",start.year[cl],"_",end.year[cl],".nc4",sep="")

           # NetCDF generation ####
            mv <- NODATA
            
            # NC file dimensions
            dim_lon <- ncdim_def("lon","degrees_east",seq(-179.75,179.75,len=360/0.5))
            #change order of latitudes
            dim_lat <- ncdim_def("lat","degrees_north",seq(89.75,-89.75,len=180/0.5))
            dim_time <- ncdim_def("time",paste("growing seasons since ",start.year[cl],"-01-01 00:00:00",sep=""),
                                  c(start.year[cl]:end.year[cl])-start.year[cl]+1,
                                  calendar = "standard")
            cat("doing",fn,"\n",names[va],longnames[va],units[va],"\n",climate[cl],clim.scenario,harmnames[ha],irrigs[ir],"\n")
            # define variable
            ncv <- ncvar_def(paste(names[va],"_",crops[cr],sep=""),units[va],list(dim_lon,dim_lat,dim_time),mv,
                             longname=paste(cropf[cr],longnames[va]),
                             compression=9)
            
            # create files
            ncf <- nc_create(fn,list(ncv))
            
            # adding variables to files
            ncatt_put(ncf,varid=0,"title","prescribed harvest dates for Ag-GRID GGCMI harmonization")
            ncatt_put(ncf,varid=0,"comment1","areas where fastest-maturing variety should be used are flagged with -99")
            #mapout <- map_cc[,360:1,2]
            
            # commenting
            ncatt_put(ncf,varid=0,"title","LPJmL5 simulations")
            ncatt_put(ncf,varid=0,"comment1","Potsdam Institute for Climate Impact Research")
            ncatt_put(ncf,varid=0,"comment2","cmueller@pik-potsdam.de")
            txt2 <- paste(model,"_",climate[cl],"_",clim.scenario,"_",harmnames[ha],"_",irrigs[ir],sep="")
            ncatt_put(ncf,varid=0,"comment3",txt2)
            #if(cl==12) ncatt_put(ncf,varid=0,"comment4","updated PGF to PGFv2.1 on March 04, 2016")
            
            # preparing data for NC files
            # having some 3D matrix: lon, lat, time
            mapo <- array(NA,dim=c(720,360,(end.year[cl]-start.year[cl]+1)))
            # loop through pixels, fill matrix
            # variables <- c("yield","pirrww","biom","aet","plant-day","maty-day","gsprcp","gsrsds","sumt")
            if(1==va) buf <- yld
            else if(2==va) buf <- piw
            else if(3==va) buf <- bim
            else if(4==va) buf <- aet
            else if(5==va) buf <- pdy
            else if(6==va) buf <- mdy
            else if(7==va) buf <- gsp
            else if(8==va) buf <- gsr
            else if(9==va) buf <- gst
            for(p in 1:ncell){
              mapo[grid$ilon[p],360-grid$ilat[p],] <- buf[p,]
            }
            
            mapo[is.na(mapo)] <- mv
            
            # writing data to NC files looping through time
            for(i in 1:length(start.year[cl]:end.year[cl])){
              ncvar_put(ncf,ncv,mapo[,,i],start=c(1,1,i),count=c(-1,-1,1))
            }
            
            # closing NC files
            nc_close(ncf)
          }#if
        }#variables
      	## foreach returns the last value/variable/result of an operation
        #message <- list("id"=mpi.comm.rank(0), "messages"=txt, "j"=cr)
        #message

      }#cr
      #print(str(parloop))
      cat("done",climate[cl],harms[ha],irrigs[ir],"\n")
      #for(job in parloop) print(paste0(job[["j"]], ": ", job[["id"]]))
      #for(job in parloop) cat(paste(str(job[["messages"]]),"\n\n",job[["messages"]]))
    }
  }
}  
print("done")

## end of script, always close worker nodes if run in parallel mode
if(parallel==T)
	closeCluster(clu)
## MPI-save version of R quit
#mpi.quit()



# nohup rsync --omit-dir-times --delete -av -e 'ssh -i ~/.ssh/dkrz' ./* b280033@vre1.dkrz.de:/gpfs_750/projects/ISI_MIP/data/AgMIP.output/LPJmL/WFDEI.GPCC/.   cmueller@cluster.pik-potsdam.de:/iplex/01/landuse/users/cmueller/IMAGE-LPJmL/.

#rsync --omit-dir-times --delete -av -e 'ssh -i ~/.ssh/dkrz' pAPSIM b380079@vre1.dkrz.de:/gpfs_750/projects/ISI_MIP/data/AgMIP.output
