/**************************************************************************************/
/**                                                                                \n**/
/**                    a  l  b  e  d  o  .  c                                      \n**/
/**                                                                                \n**/
/**     Computation of albedo as described by Bob Gallimore (in an e-mail)         \n**/
/**     and used by him in the coupling of FOAM to LPJ.                            \n**/
/**                                                                                \n**/
/** (C) Potsdam Institute for Climate Impact Research (PIK), see COPYRIGHT file    \n**/
/** authors, and contributors see AUTHORS file                                     \n**/
/** This file is part of LPJmL and licensed under GNU AGPL Version 3               \n**/
/** or later. See LICENSE file or go to http://www.gnu.org/licenses/               \n**/
/** Contact: https://github.com/PIK-LPJmL/LPJmL                                    \n**/
/**************************************************************************************/

#include "lpj.h"

static const Real c_albAntarctica  = 0.7;
static const Real c_roughness      = 0.06; /* Roughness height of vegetation below the canopy. Source: FOAM/LPJ */

static Real albedo_stand(const Stand *stand)
{
  int p;
  Real frsg;  /* Snow cover of the ground below the canopy (0-1) */
  Real HS;    /* Height of the Snow (m) */
  Real albstot;/* Sum of all albs over all pfts */
  Real fbare;  /* Fraction bare soil */
  const Pft *pft;
  Real test;
  if (stand->soil.par->type == ICE /* ICE */)
  {
    if (stand->cell->coord.lat<-60)
      return c_albAntarctica;
    return c_albsnow;
  }
  HS = c_watertosnow * (stand->soil.snowpack/1000.0); /* mm -> m */
  frsg = HS / (HS+0.5*c_roughness); /* If HS is large, frsg will be close to 1. For small values of HS, frsg will be less than 1, because then a significant amount of the snow is captured by the canopy */
  fbare = 0.0;
  albstot = 0.0;

  foreachpft(pft,p,&stand->pftlist)
  {
    albstot += pft->par->albedo(pft,HS,frsg); /* call PFT-specific albedo function */
    fbare += pft->fpc;
  } /* pft loop */
  fbare = max((1-fbare),0.0);

  test = albstot + fbare * (frsg * c_albsnow + (1-frsg) * c_albsoil);

#ifndef NDEBUG
  if (test > 1 || test < 0 || isnan(test))
  {
    printf("WARNING albedo out of bounds: %g fbare=%g albstot=%g\n", test, fbare, albstot);
    foreachpft(pft,p,&stand->pftlist)
    {
      printf("pft %s fpc %g phen %g\n", pft->par->name, pft->fpc, pft->phen);
    }
  }
#endif

  return  test;

} /* end of albedo_stand() */

Real albedo(const Cell *cell)
{
  const Stand *stand;
  int s;
  Real a;
  a=0;
  foreachstand(stand,s,cell->standlist)
    a+=albedo_stand(stand)*stand->frac;
  return a;
} /* of 'albedo' */
